#!/usr/bin/python
from albatross import SimpleSessionApp
from albatross.cgiapp import Request


class ListPage:

    def page_display(self, ctx):
        ctx.locals.data = range(100)
        ctx.run_template('paginate.html')


app = SimpleSessionApp(base_url='paginate.py',
                       template_path='.',
                       start_page='start',
                       secret='-=-secret-=-',
                       session_appid='paginate')
app.register_page('start', ListPage())


if __name__ == '__main__':
    app.run(Request())
