import sys
sys.path.insert(0, '..')
import install
from install import Installer

i = Installer('alsamp/popview4', cgi = 1)
i.add_exec('popview.py')
i.add_module('popviewlib.py')
i.add_module('login.py')
i.add_html('login.html')
i.add_module('list.py')
i.add_html('list.html')
i.add_module('detail.py')
i.add_html('detail.html')
i.install()
