<al-expand name="doc">
  <al-setarg name="title">error page</al-setarg>
  <al-if expr="page == 'oops.html'">
   You actually requested the error page!
  <al-else>
   Sorry, the page <font color="red"><al-value expr="page"></font>
   does not exist.
  </al-if>
</al-expand>
