#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
#
# Basic functionality tests - save and restore a single context
#
# $Id: concurrent.py,v 1.3 2002/07/30 07:33:51 andrewm Exp $
#

import unittest

class BasicCase(unittest.TestCase):
    def check_file(self):
        pass

    def check_server(self):
        pass

    def check_hiddenfield(self):
        pass

class BasicSuite(unittest.TestSuite):
    # add modules with tests here
    test_list = [
        "check_file",
        "check_server",
        "check_hiddenfield",
    ]

    def __init__(self):
        unittest.TestSuite.__init__(self, map(BasicCase, self.test_list))

def suite():
    return BasicSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='BasicSuite')
