import sys
sys.path.insert(0, '../..')
import install
from install import Installer

i = Installer('alsamp/content1', cgi = 1)
i.add_exec('content.py')
i.add_html('main.html', 'templ/')
i.add_html('oops.html', 'templ/')
i.add_html('other.html', 'templ/')
i.install()
