<al-input type="checkbox" name="nums" value="one" expr="selected" list whitespace>
<al-input type="checkbox" name="nums" value="two" expr="selected" list whitespace>
<al-input type="checkbox" name="nums" value="three" expr="selected" list whitespace>
<al-input type="checkbox" name="nums" valueexpr="vexpr" expr="selected" list whitespace>
