import sys
sys.path.insert(0, '..')
import install
from install import Installer

i = Installer('alsamp/images', doc = 1)
i.add_image('open.png')
i.add_image('close.png')
i.add_image('ellipsis.png')
i.install()
