import sys
sys.path.insert(0, '..')
import install
from install import Installer

i = Installer('alsamp/mpperf', apache = 1)
i.add_exec('mpperf.py')
i.add_module('main.py')
i.add_html('main.html')
i.add_data('htaccess', '.htaccess')
i.install()
