<al-form whitespace>
 <table>
  <al-tree iter="n" expr="tree" lazy>
   <tr>
    <al-if expr="n.depth() != 0">
     <td width="16" colspan="<al-value expr="n.depth()">" nowrap></td>
    </al-if>
    <al-if expr="n.has_children()">
     <td width="16">
      <al-if expr="n.is_open()">
       <al-input type="image" src="open.png" width="13" height="13" border="0" treefold="n">
      <al-else>
       <al-input type="image" src="close.png" width="13" height="13" border="0" treefold="n">
      </al-if>
     </td>
    <al-else>
     <td width="16" nowrap></td>
    </al-if>
    <td width="16"><al-input type="checkbox" treeselect="n"></td>
    <td colspan="999" nowrap><al-value expr="n.value().name"></td>
   </tr>
  </al-tree>
 </table>
</al-form>
