<html>
 <head><title>I think I can, I think I can</title></head>
 <body>
  <p>Calculation is in progress, please stand by.
  <p>
   <al-for iter="n" expr="process">
    <al-flush>
    Stage: <al-value expr="n.value()"><br>
   </al-for>
  <p>All done!
 </body>
</html>
