<html>
 <head>
  <title>Simple Content Management - error page.</title>
 </head>
 <body>
  <h1>Simple Content Management - error page</h1>
  <hr noshade>
  <al-if expr="page == 'oops.html'">
   You actually requested the error page!
  <al-else>
   Sorry, the page <font color="red"><al-value expr="page"></font>
   does not exist.
  </al-if>
 </body>
</html>
