#!/usr/bin/python
from albatross import RandomModularSessionApp
from albatross.cgiapp import Request


app = RandomModularSessionApp(base_url='randompage.py',
                              page_path='pages',
                              start_page='tree',
                              secret='-=-secret-=-',
                              session_appid='random')


if __name__ == '__main__':
    app.run(Request())
