from albatross import ModularApp
from albatross.apacheapp import Request


app = ModularApp(base_url='mpperf',
                 module_path='-=-install_dir-=-',
                 template_path='-=-install_dir-=-',
                 start_page='main',
                 secret='-=-secret-=-')


def handler(req):
    return app.run(Request(req))
