<al-macro name="one-arg">
 xxx <al-usearg name="arg1"> <al-usearg name="arg2"> xxx
</al-macro>
<al-expand name="one-arg">
 <al-setarg name="arg1">first</al-setarg>
 <al-setarg name="arg2">second</al-setarg>
</al-expand>
