import sys
sys.path.insert(0, '../..')
import install
from install import Installer

i = Installer('alsamp/form2', cgi = 1)
i.add_exec('form.py')
i.add_html('form.html')
i.add_html('form-display.html')
i.install()
