import sys
sys.path.insert(0, '..')
import install
from install import Installer

i = Installer('alsamp/random', cgi = 1)
i.add_exec('randompage.py')
i.add_exec('pages/tree.py', 'pages/')
i.add_html('pages/tree.html', 'pages/')
i.add_exec('pages/paginate.py', 'pages/')
i.add_html('pages/paginate.html', 'pages/')
i.add_module('utils.py')
i.install()
