import sys
sys.path.insert(0, '..')
import install
from install import Installer

i = Installer('alsamp/sybase', apache = 1)
i.add_exec('sybase.py')
i.add_html('macros.html')
i.add_html('table-desc.html')
i.add_html('table-list.html')
i.add_html('table-view.html')
i.add_html('traceback.html')
i.add_data('htaccess', '.htaccess')
i.install()
